#!/usr/bin/python
#
# Copyright 2011 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Creates a default OAuth handler."""

__author__ = 'api.kwinter@gmail.com (Kevin Winter)'

import urllib
import urlparse
import oauth2 as oauth
from OAuthHandler import OAuthHandler


class PythonOAuth2OAuthHandler(OAuthHandler):
  """An OAuth handler that uses python-oauth2.

  Library included from: https://github.com/simplegeo/python-oauth2.
  """

  def GetRequestToken(self, credentials, scope, server=None, callbackurl=None,
                      applicationname=None):
    """Gets a request token.

    Args:
      credentials: dict The credentials, including the consumer key and consumer
                   secret.
      scope: str The scope of the application to authorize.
      server: str Optional OAuth server to make the request against.
      callbackurl: str Optional callback url.
      applicationname: str Optional name of the application to display on the
                       authorization redirect page.
    Returns:
      dict The credentials, including the consumer key, consumer secret, token
           and token secret.
    """
    consumer = oauth.Consumer(credentials['oauth_consumer_key'],
                              credentials['oauth_consumer_secret'])
    signature_method = oauth.SignatureMethod_HMAC_SHA1()

    params = {
        'scope': scope,
        'oauth_version': '1.0a'
    }
    if applicationname: params['xoauth_displayname'] = applicationname
    if callbackurl:
      params['oauth_callback'] = callbackurl
    else:
      params['oauth_callback'] = self.DEFAULT_CALLBACK_URL
    endpoint = self.GetRequestEndpoint(server)

    request = oauth.Request.from_consumer_and_token(consumer, None, 'GET',
                                                    endpoint, params)
    request.sign_request(signature_method, consumer, None)

    token = self.GetTokenFromUrl(request.to_url())
    credentials['oauth_token'] = token.key
    credentials['oauth_token_secret'] = token.secret
    return credentials

  def GetAccessToken(self, credentials, verifier, server=None):
    """Gets the access token for an authorized request token.

    Args:
      credentials: dict The credentials, including the consumer key, consumer
                   secret, token, and token secret.
      verifier: str The OAuth verifier code returned with the callback.
      server: str Optional OAuth server to make the request against.
    Returns:
      str The credentials, including the consumer key, consumer secret, token,
          and token secret.
    """
    consumer = oauth.Consumer(credentials['oauth_consumer_key'],
                              credentials['oauth_consumer_secret'])
    token = oauth.Token(credentials['oauth_token'],
                        credentials['oauth_token_secret'])
    signature_method = oauth.SignatureMethod_HMAC_SHA1()

    if verifier:
      params = {
          'oauth_verifier': verifier,
          'oauth_version': '1.0a'
      }
    else:
      params = None
    endpoint = self.GetAccessEndpoint(server)

    request = oauth.Request.from_consumer_and_token(consumer, token, 'GET',
                                                    endpoint, params)
    request.sign_request(signature_method, consumer, token)

    token = self.GetTokenFromUrl(request.to_url())
    credentials['oauth_token'] = token.key
    credentials['oauth_token_secret'] = token.secret
    return credentials

  def GetSignedRequestParameters(self, credentials, url):
    """Gets the signed OAuth parameters required to make a request for the URL.

    Args:
      credentials: dict The credentials, including the consumer key, consumer
                        secret, token, and token secret.
      url: str The URL the request will be made against.
    Returns:
      dict An array of OAuth parameter names to values.
    """
    consumer = oauth.Consumer(credentials['oauth_consumer_key'],
                              credentials['oauth_consumer_secret'])
    token = oauth.Token(credentials['oauth_token'],
                        credentials['oauth_token_secret'])
    signature_method = oauth.SignatureMethod_HMAC_SHA1()

    params = {
        'oauth_version': '1.0a'
    }

    request = oauth.Request.from_consumer_and_token(consumer, token, 'POST',
                                                    url, params)
    request.sign_request(signature_method, consumer, token)
    return dict(request)

  def GetTokenFromUrl(self, url):
    """Makes and HTTP request to the given URL and extracts the OAuth token.

    Args:
      url: str The URL to make the request to.
    Returns:
      Token The returned token.
    """
    response = urllib.urlopen(url).read()
    return self.GetTokenFromQueryString(response)

  def GetTokenFromQueryString(self, querystring):
    """Parses a query string and extracts the OAuth token.

    Args:
      querystring: str The query string.
    Returns:
      Token The returned token.
    """
    values = urlparse.parse_qs(querystring)
    return oauth.Token(values['oauth_token'][0],
                       values['oauth_token_secret'][0])



#!/usr/bin/python
#
# Copyright 2011 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Creates custom handler for OAuth messiness."""

__author__ = 'api.kwinter@gmail.com (Kevin Winter)'

import urllib
import urlparse


class OAuthHandler(object):
  """Defines a base, abstract class that handles the messiness of OAuth."""

  DEFAULT_CALLBACK_URL = 'oob'

  REQUEST_ENDPOINT = 'https://www.google.com/accounts/OAuthGetRequestToken'
  AUTHORIZE_ENDPOINT = 'https://www.google.com/accounts/OAuthAuthorizeToken'
  ACCESS_ENDPOINT = 'https://www.google.com/accounts/OAuthGetAccessToken'

  def GetRequestToken(self, credentials, scope, server=None, callbackurl=None,
                      applicationname=None):
    """Gets a request token.

    Must be overridden by implementors.

    Args:
      credentials: dict The credentials, including the consumer key and consumer
                   secret.
      scope: str The scope of the application to authorize.
      server: str Optional OAuth server to make the request against.
      callbackurl: str Optional callback url.
      applicationname: str Optional name of the application to display on the
                       authorization redirect page.
    Returns:
      dict The credentials, including the consumer key, consumer secret, token
           and token secret.
    """
    raise NotImplementedError

  def GetAuthorizationUrl(self, credentials, server=None):
    """Gets the authorization URL for a request token.

    Args:
      credentials: dict The credentials, including the token.
      server: str Optional OAuth server to generate the URL for.
    Returns:
      str An authorization URL to redirect users to.
    """
    endpoint = self.GetAuthorizeEndpoint(server, None)
    params = {
        'oauth_token': credentials['oauth_token']
    }
    return self.AddParamsToUrl(endpoint, params)

  def GetAccessToken(self, credentials, verifier, server=None):
    """Gets the access token for an authorized request token.

    Must be overridden by implementors.

    Args:
      credentials: dict The credentials, including the consumer key, consumer
                   secret, token, and token secret.
      verifier: str The OAuth verifier code returned with the callback.
      server: str Optional OAuth server to make the request against.
    Returns:
      str The credentials, including the consumer key, consumer secret, token,
          and token secret.
    """
    raise NotImplementedError

  def GetSignedRequestParameters(self, credentials, url):
    """Gets the signed OAuth parameters required to make a request for the URL.

    Must be overridden by implementors.

    Args:
      credentials: dict The credentials, including the consumer key, consumer
                        secret, token, and token secret.
      url: str The URL the request will be made against.
    Returns:
      dict An array of OAuth parameter names to values.
    """
    raise NotImplementedError

  def FormatParametersForUrl(self, params):
    """Formats OAuth parameters for use in a URL.

    For example: param1=value1&param2=value2.

    Args:
      params: dict The OAuth parameters.
    Returns:
      str The parameters formatted for use in a URL.
    """
    return urllib.urlencode(params)

  def FormatParametersForHeader(self, params):
    """Formats OAuth parameters for use in an HTTP header.

    For example: param1="value1", param2="value2"

    Args:
      params: dict The OAuth parameters.
    Returns:
      str The parameters formatted for use in an HTTP header.
    """
    return ', '.join('%s="%s"' %
                     (key, urllib.quote_plus(value))
                     for key, value in params.items())

  def GetRequestEndpoint(self, server=None, params=None):
    """Gets the request endpoint using the given server and parameters.

    Args:
      server: str Optional OAuth server to use for the endpoint.
      params: dict Optional params to include in the endpoint.
    Returns:
      str The request endpoint.
    """
    return self.GetEndpoint(self.__class__.REQUEST_ENDPOINT, server, params)

  def GetAuthorizeEndpoint(self, server=None, params=None):
    """Gets the authorization endpoint using the given server and parameters.

    Args:
      server: str Optional OAuth server to use for the endpoint.
      params: dict Optional params to include in the endpoint.
    Returns:
      str The authorization endpoint.
    """
    return self.GetEndpoint(self.__class__.AUTHORIZE_ENDPOINT, server, params)

  def GetAccessEndpoint(self, server=None, params=None):
    """Gets the access endpoint using the given server and parameters.

    Args:
      server: str Optional OAuth server to use for the endpoint.
      params: dict Optional params to include in the endpoint.
    Returns:
      str The access endpoint.
    """
    return self.GetEndpoint(self.__class__.ACCESS_ENDPOINT, server, params)

  def GetEndpoint(self, endpoint, server=None, params=None):
    """Gets an endpoint using the given server and parameters.

    Args:
      endpoint: str The base endpoint URL to use.
      server: str Optional OAuth server to use for the endpoint.
      params: dict Optional params to include in the endpoint.
    Returns:
      str The endpoint.
    """
    endpoint = self.AddParamsToUrl(endpoint, params)
    if server:
      endpoint = self.ReplaceServerInUrl(endpoint, server)
    return endpoint

  def ReplaceServerInUrl(self, url, server):
    """Replaces the protocol and server portion of a URL with another.

    Args:
      url: str The full URL.
      server: str The protocol and server to replace with.
    Returns:
      str The URL with the protocol and server replaced.
    """
    _, _, path, params, query, fragment = urlparse.urlparse(url)
    newscheme, newhost, _, _, _, _ = urlparse.urlparse(server)
    return urlparse.urlunparse((newscheme, newhost, path, params, query,
                                fragment))

  def AddParamsToUrl(self, url, params):
    """Adds parameters to a URL.

    Args:
      url: str The URL.
      params: dict The parameters to add.
    Returns:
      str The new URL with the parameters added.
    """
    if not params: return url
    paramstring = self.FormatParametersForUrl(params)
    scheme, host, path, curparams, query, fragment = urlparse.urlparse(url)
    if query:
      query = query + '&' + paramstring
    else:
      query = paramstring
    return urlparse.urlunparse((scheme, host, path, curparams, query, fragment))




#!/usr/bin/python
#
# Copyright 2010 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Classes for handling errors."""

__author__ = 'api.sgrinberg@gmail.com (Stan Grinberg)'

from adspygoogle.adwords import ERROR_TYPES
from adspygoogle.common.Errors import DetailError
from adspygoogle.common.Errors import Error


class AdWordsError(Error):

  """Implements AdWordsError.

  Responsible for handling error.
  """
  pass


class AdWordsDetailError(DetailError):

  """Implements AdWordsDetailError.

  Responsible for handling detailed ApiException error.
  """

  def __init__(self, **error):
    DetailError.__init__(self)
    self.__error = error
    for key in self.__error:
      self.__dict__.__setitem__(key, self.__error[key])

  def __call__(self):
    return (self.__error,)


class AdWordsApiError(AdWordsError):

  """Implements AdWordsApiError.

  Responsible for handling API exception.
  """

  def __init__(self, fault):
    (self.__fault, self.fault_code, self.fault_string) = (fault, '', '')
    if 'faultcode' in fault: self.fault_code = fault['faultcode']
    if 'faultstring' in fault: self.fault_string = fault['faultstring']

    (self.code, error_msg, self.trigger, self.errors) = (-1, '', '', [])
    if 'detail' in fault and fault['detail']:
      if 'code' in fault['detail']: self.code = int(fault['detail']['code'])
      if 'trigger' in fault['detail']: self.trigger = fault['detail']['trigger']
      if 'message' in fault['detail']: error_msg = fault['detail']['message']
    if not error_msg: error_msg = self.fault_string

    errors = [None]
    if 'detail' in fault and fault['detail'] and 'errors' in fault['detail']:
      errors = fault['detail']['errors']
    elif 'detail' not in fault or not fault['detail']:
      errors[0] = {}
    else:
      errors[0] = fault['detail']
    if isinstance(errors, list):
      for error in errors:
        # Keys need to be of type str not unicode.
        error_dct = dict([(str(key), value) for key, value in error.items()])
        self.errors.append(AdWordsDetailError(**error_dct))
    else:
      # Keys need to be of type str not unicode.
      error_dct = dict([(str(key), value) for key, value in errors.items()])
      self.errors.append(AdWordsDetailError(**error_dct))
    AdWordsError.__init__(self, error_msg)

  def __call__(self):
    return (self.__fault,)


class AdWordsRequestError(AdWordsApiError):

  """Implements AdWordsRequestError.

  Responsible for handling request error."""

  pass


class AdWordsGoogleInternalError(AdWordsApiError):

  """Implements AdWordsGoogleInternalError.

  Responsible for handling Google internal error.
  """

  pass


class AdWordsAuthenticationError(AdWordsApiError):

  """Implements AdWordsAuthenticationError.

  Responsible for handling authentication error.
  """

  pass


class AdWordsAccountError(AdWordsApiError):

  """Implements AdWordsAccountError.

  Responsible for handling account error.
  """

  pass


class AdWordsWebpageError(AdWordsApiError):

  """Implements AdWordsWebpageError.

  Responsible for handling webpage error.
  """

  pass


class AdWordsBillingError(AdWordsApiError):

  """Implements AdWordsBillingError.

  Responsible for handling billing error.
  """

  pass


class AdWordsReportError(AdWordsError):

  def __init__(self, http_code, error_type, trigger, field_path):
    self.http_code = http_code
    self.type = error_type
    self.trigger = trigger
    self.field_path = field_path
    message = ('HTTP code: %s, type: \'%s\', trigger: \'%s\', '
               'field path: \'%s\'' %
               (http_code, error_type, trigger, field_path))
    super(AdWordsReportError, self).__init__(message)


# Map error codes and types to their corresponding classes.
ERRORS = {}
ERROR_CODES = [x for x in xrange(0, 208)]
for index in ERROR_CODES+ERROR_TYPES:
  if (((index >= 1 and index <= 10) or (index >= 12 and index <= 17) or
       (index >= 19 and index <= 42) or (index >= 43 and index <= 49) or
       index == 51 or index == 54 or (index >= 57 and index <= 59) or
       (index >= 61 and index <= 63) or (index >= 70 and index <= 83) or
       (index >= 87 and index <= 94) or index == 96 or index == 97 or
       index == 99 or index == 112 or index == 115 or index == 116 or
       (index >= 120 and index <= 125) or index == 127 or index == 128 or
       index == 131 or index == 133 or index == 134 or index == 137 or
       index == 138 or (index >= 140 and index <= 142) or
       (index >= 144 and index <= 147) or index == 149 or index == 153 or
       (index >= 156 and index <= 158) or (index >= 170 and index <= 174) or
       index == 176 or index == 177 or index == 186 or index == 188 or
       index == 190 or index == 206 or index == 207) or
      (index in ('AdError', 'AdExtensionError', 'AdExtensionOverrideError',
                 'AdGroupAdError', 'AdGroupCriterionError',
                 'AdGroupServiceError', 'AdParamError', 'AdParamPolicyError',
                 'AlertError', 'ApiError', 'ApiUsageError', 'AudioError',
                 'BidLandscapeServiceError', 'BiddingError',
                 'BiddingTransitionError', 'BudgetError', 'BulkMutateJobError',
                 'CampaignAdExtensionError',
                 'CampaignCriterionError', 'CampaignError',
                 'CollectionSizeError', 'ConversionTrackingError',
                 'CriterionError', 'CriterionPolicyError', 'CurrencyCodeError',
                 'CustomerSyncError', 'DataError', 'DateError', 'DistinctError',
                 'ExperimentServiceError', 'GeoLocationError', 'IdError',
                 'ImageError', 'JobError', 'MatchesRegexError', 'MediaError',
                 'NewEntityCreationError', 'NotEmptyError', 'NullError',
                 'OperatorError', 'OpportunityError', 'PagingError',
                 'PolicyViolationError', 'RangeError', 'ReadOnlyError',
                 'RegionCodeError', 'RejectedError', 'ReportDefinitionError',
                 'RequestError', 'RequiredError', 'SelectorError',
                 'ServicedAccountError', 'SettingError', 'SizeLimitError',
                 'StatsQueryError', 'StringLengthError', 'TargetError',
                 'TargetingIdeaError', 'TrafficEstimatorError', 'UserListError',
                 'VideoError'))):
    ERRORS[index] = AdWordsRequestError
  elif ((index == 0 or index == 18 or index == 55 or index == 60 or
         index == 95 or index == 98 or index == 117 or index == 143 or
         index == 155) or
        (index in ('InternalApiError', 'DatabaseError', 'QuotaCheckError',
                   'QuotaError', 'QuotaExceededError', 'RateExceededError'))):
    ERRORS[index] = AdWordsGoogleInternalError
  elif (((index >= 84 and index <= 86) or index == 111 or index == 119 or
         index == 129 or index == 139 or (index >= 162 and index <= 165) or
         index == 183 or index == 189) or
        (index in ('ClientTermsError', 'NotWhitelistedError'))):
    ERRORS[index] = AdWordsAccountError
  elif (index >= 100 and index <= 105):
    ERRORS[index] = AdWordsWebpageError
  elif (index == 50 or index == 52 or index == 53 or index == 106 or
         index == 107 or index == 109 or index == 110 or index == 114 or
         index == 118 or index == 130 or index == 132) or (index in ()):
    ERRORS[index] = AdWordsBillingError
  elif ((index == 166 or index == 184) or
        ('AuthenticationError', 'AuthorizationError')):
    ERRORS[index] = AdWordsAuthenticationError
